@@ -1,7 +1,11 @@
+from selenium.webdriver.support import expected_conditions as EC
+from selenium.webdriver.support.ui import WebDriverWait
+from selenium.webdriver.common.by import By
+
+import undetected_chromedriver as uc
 import requests
 import uuid
 import json
-import re
 
 
 class ChatGPT:
@@ -11,9 +15,7 @@ class ChatGPT:
 
     def __init__(
         self,
-        session_token: str = None,
-        email: str = None,
-        password: str = None,
+        session_token: str,
         conversation_id: str = None,
         parent_id: str = None,
         proxy: str = None,
@@ -22,9 +24,7 @@ class ChatGPT:
         Initialize the ChatGPT class\n
         Either provide a session token or email and password\n
         Parameters:
-        - session_token: (optional) Your session token in cookies named as `__Secure-next-auth.session-token` from https://chat.openai.com/chat
-        - email: (optional) Your OpenAI email
-        - password: (optional) Your OpenAI password
+        - session_token: Your session token in cookies named as `__Secure-next-auth.session-token` from https://chat.openai.com/chat
         - conversation_id: (optional) The conversation ID if you want to continue a conversation
         - parent_id: (optional) The parent ID if you want to continue a conversation
         - proxy: (optional) The proxy to use, in URL format (i.e. `https://ip:port`)
@@ -34,6 +34,8 @@ class ChatGPT:
             self.parent_id = parent_id
         else:
             self.parent_id = str(uuid.uuid4())
+
+        self.proxy = proxy
         self.proxies = {'http': proxy, 'https': proxy} if proxy else {}
         self.headers = {
             'accept': '*/*',
@@ -41,13 +43,6 @@ class ChatGPT:
             'accept-language': 'en-US;q=0.9,en;q=0.8',
             'origin': 'https://chat.openai.com',
             'referer': 'https://chat.openai.com/chat',
-            'sec-ch-ua': '"Not?A_Brand";v="8", "Chromium";v="108", "Microsoft Edge";v="108"',
-            'sec-ch-ua-mobile': '?0',
-            'sec-ch-ua-platform': '"Windows"',
-            'sec-fetch-dest': 'empty',
-            'sec-fetch-mode': 'cors',
-            'sec-fetch-site': 'same-origin',
-            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36 Edg/108.0.1462.46',
             'x-openai-assistant-app-id': '',
         }
         self.session = requests.Session()
@@ -55,98 +50,39 @@ class ChatGPT:
         self.session.proxies = self.proxies
 
         self.session_token = session_token
-        if not self.session_token:
-            if not email or not password:
-                raise ValueError('No session token or login credentials are provideddd')
-            self._login(email, password)
-        else:
-            self.headers[
-                'Cookie'
-            ] = f'__Secure-next-auth.session-token={self.session_token}'
-            self.refresh_auth()
+        options = uc.ChromeOptions()
 
-    def _login(self, email: str, password: str) -> str:
+        if self.proxy:
+            options.add_argument(f'--proxy-server={self.proxy}')
+        self.driver = uc.Chrome(options=options)
+        self.refresh_auth(init=True)
+
+    def refresh_auth(self, init: bool = False) -> None:
         '''
-        Login to OpenAI\n
+        Refresh the session's authorization
         Parameters:
-        - email: Your OpenAI email
-        - password: Your OpenAI password\n
-        Returns the session token
+        - init: (optional) Whether to initialize the session
         '''
-        self.session.headers = self.headers
-        self.session.proxies = self.proxies
+        if init:
+            self.driver.get('https://chat.openai.com/api/auth/session')
+            self.headers['user-agent'] = self.driver.execute_script(
+                'return navigator.userAgent'
+            )
+            self.driver.add_cookie(
+                {
+                    'name': '__Secure-next-auth.session-token',
+                    'value': self.session_token,
+                }
+            )
 
-        # Get the CSRF token
-        resp = self.session.get('https://chat.openai.com/api/auth/csrf')
-        if resp.status_code != 200:
-            raise ValueError(f'Status code {resp.status_code}: {resp.text}')
-        csrf_token = resp.json()['csrfToken']
-
-        # Get state
-        resp = self.session.post(
-            'https://chat.openai.com/api/auth/signin/auth0?prompt=login',
-            data={'callbackUrl': '/', 'csrfToken': csrf_token, 'json': 'true'},
+        self.driver.get('https://chat.openai.com/api/auth/session')
+        WebDriverWait(self.driver, 10).until(
+            EC.presence_of_element_located((By.TAG_NAME, 'pre'))
         )
-        if resp.status_code != 200:
-            raise ValueError(f'Status code {resp.status_code}: {resp.text}')
-        redirect_url = resp.json()['url']
-
-        # Redirect to auth0 /login/identifier
-        resp = self.session.get(redirect_url)
-        if resp.status_code != 200:
-            raise ValueError(f'Status code {resp.status_code}: {resp.text}')
-        if '<img alt="captcha"' in resp.text:
-            raise ValueError('Captcha detected')
-        pattern = r'<input type="hidden" name="state" value="(.*)" \/>'
-        results = re.findall(pattern, resp.text)
-        if not results:
-            raise ValueError(f'Could not get state: {resp.text}')
-        state = results[0]
-
-        # Post email
-        resp = self.session.post(
-            f'https://auth0.openai.com/u/login/identifier?state={state}',
-            data={
-                'state': state,
-                'username': email,
-                'js-available': 'false',
-                'webauthn-available': 'true',
-                'is-brave': 'false',
-                'webauthn-platform-available': 'false',
-                'action': 'default',
-            },
-        )
-        if resp.status_code != 200:
-            raise ValueError(f'Status code {resp.status_code}: {resp.text}')
-
-        # Post password
-        resp = self.session.post(
-            f'https://auth0.openai.com/u/login/password?state={state}',
-            data={
-                'state': state,
-                'username': email,
-                'password': password,
-                'action': 'default',
-            },
-        )
-        if resp.status_code != 200:
-            raise ValueError(f'Status code {resp.status_code}: {resp.text}')
-
-        # Get session token in CookieJar
-        cookies = self.session.cookies.get_dict()
-        if '__Secure-next-auth.session-token' not in cookies:
-            raise ValueError(f'Could not get session token: {cookies}')
-        return cookies['__Secure-next-auth.session-token']
-
-    def refresh_auth(self) -> None:
-        '''
-        Refresh the authorization token
-        '''
-        resp = self.session.get('https://chat.openai.com/api/auth/session')
-        if resp.status_code != 200:
-            raise ValueError(f'Status code {resp.status_code}: {resp.text}')
-
-        data = resp.json()
+        for cookie in self.driver.get_cookies():
+            self.session.cookies.set(cookie['name'], cookie['value'])
+        resp = self.driver.find_element(By.TAG_NAME, 'pre').text
+        data = json.loads(resp)
         if not data:
             raise ValueError('Invalid session token')
         access_token = data['accessToken']
